local _, addon = ...
local sm3 = nil
local sink = nil

local L = nil

local validTypes = {
	statusbar = true,
	border = true,
	font = true,
	sound = true,
	background = true,
	sink = true,
}

local function attemptToLoadSink()
	if sink then return end
	sink = LibStub("LibSink-2.0", true)
	if sink and SurfaceControlDB.sink then
		sink:SetSinkOverride(SurfaceControlDB.sink)
	end
end

local function attemptToLoadLSM()
	if sm3 then return end
	sm3 = LibStub("LibSharedMedia-3.0", true)
	if sm3 then
		for k, v in pairs(SurfaceControlDB) do
			if k ~= "sink" then
				sm3:SetGlobal(k, v)
			end
		end
	end
end

local function slashHandler(input)
	attemptToLoadSink()
	attemptToLoadLSM()

	local foo, bar = strsplit(" ", input)
	if not validTypes[foo] then
		print(L["Accepted types include statusbar, border, font, sound, background and sink."])
		return
	end
	if foo == "sink" then
		if not sink then
			print(L["Couldn't find %s."]:format("LibSink-2.0"))
			return
		end
		if bar and sink.handlers and not sink.handlers[bar] then
			print(L["The given %s, %q, was not found in %s, or is not valid."]:format(foo, tostring(bar), "LibSink-2.0"))
			return
		end
		if not bar then
			SurfaceControlDB.sink = nil
			print(L["Sink override will be reset the next time you reload your interface."])
		else
			SurfaceControlDB.sink = bar
			sink:SetSinkOverride(bar)
			print(L["%s %q override set to %q."]:format("LibSink-2.0", foo, bar))
		end
	else
		if not sm3 then
			print(L["Couldn't find %s."]:format("LibSharedMedia-3.0."))
			return
		end
		if not sm3:IsValid(foo, bar) then
			print(L["The given %s, %q, was not found in %s, or is not valid."]:format(foo, tostring(bar), "LibSharedMedia-3.0"))
			return
		end
		sm3:SetGlobal(foo, bar)
		if not bar then
			SurfaceControlDB[foo] = nil
			print(L["LSM3 override for %q has been reset."]:format(foo))
		else
			SurfaceControlDB[foo] = bar
			print(L["%s %q override set to %q."]:format("LibSharedMedia-3.0", foo, bar))
		end
	end
end

local eventFrame = CreateFrame("Frame")
eventFrame:SetScript("OnEvent", function(self, event, arg1)
	if event == "ADDON_LOADED" then
		if arg1 == "!SurfaceControl" then
			SurfaceControlDB = SurfaceControlDB or {}
			L = addon:LoadTranslations(GetLocale())
			addon.LoadTranslations = nil
		end

		attemptToLoadSink()
		attemptToLoadLSM()
		if sink and sm3 then
			self:UnregisterEvent("ADDON_LOADED")
		end
	else
		SlashCmdList.SurfaceControl = slashHandler
		SLASH_SurfaceControl1 = "/surfacecontrol"

		attemptToLoadSink()
		attemptToLoadLSM()
	end
end)
eventFrame:RegisterEvent("PLAYER_LOGIN")
eventFrame:RegisterEvent("ADDON_LOADED")

